﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

// neu
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics; // run

namespace Hexeditor
{
    public class HexPanel:Panel
    {
        VScrollBar _vScrollBar1;
        HexFile _file;
        String _sFilename;
        String _sFileShort;

        int _fontsize = 10;
        double _leftBorder = 0;
        int _rightBorder = 100;
        double _wFF = 10;  // Mitte für einen Buchstaben
        int _rowHeight = 1;
        int _offset = 0;        // Aktueller Offset top
        int _blockRowCount=1;   // Wieviele Zeilen pro Panel
        int _colCount=16;       // Anzahl der Zeichen pro Zeile
        long _filesize = 0;
        Font _drawFont1 = null;
        Font _drawFontCenter = null;
        bool _bFirst = true;

        SolidBrush _drawBrush1 = new SolidBrush(Color.Black);
        SolidBrush _drawBrushCenter = new SolidBrush(Color.Blue);


        public HexPanel(Color color, int fontsize, int colSize, String sFilename)
        {
            base.BackColor = color;
            

            _sFilename = sFilename;
            _file = new HexFile(sFilename);
            _sFileShort = basis.getFilename(sFilename);
            _fontsize = fontsize;

            _filesize = _file.fileSize;
            _vScrollBar1 = new System.Windows.Forms.VScrollBar();
            Controls.Add(_vScrollBar1);
            _vScrollBar1.Dock = DockStyle.Right;
            this.Paint += new System.Windows.Forms.PaintEventHandler(this.panel_Paint);
            _vScrollBar1.Scroll += new System.Windows.Forms.ScrollEventHandler(vScrollBar1_Scroll);
            this.DoubleBuffered = false;
            
            initGrafik();
            _colCount = colSize;
            int lastrow = (int)Math.Ceiling((double)_filesize / _colCount)-1;
            
            _vScrollBar1.Minimum = 0;
            _vScrollBar1.Maximum = lastrow;  // Anzahl der Zeilen
            _vScrollBar1.SmallChange = 1;
            _vScrollBar1.LargeChange = 25;  // Anzahl der Zeilen pro Fenster, gschätzt
            _vScrollBar1.Value = 0;
            
        }

        ~HexPanel()
        {
            _drawFont1.Dispose();
            _drawBrush1.Dispose();
            _drawBrushCenter.Dispose();
        }

        public Color bgColor {
            set
            {
                base.BackColor = value;
            }
        }
        public int fontSize
        {
            set
            {
                _fontsize = value;
                _drawFont1.Dispose();
                _bFirst = true;       
                initGrafik();
                Invalidate();
            }
        }
        public int colSize
        {
            set
            {
                _colCount = value;
                _bFirst = true;               
                _vScrollBar1.Maximum = (int)Math.Ceiling((double)_filesize / _colCount) - 1;
                StringBuilder sb = new StringBuilder();
                Graphics g = this.CreateGraphics();
                String s = "F F";
                SizeF sf = g.MeasureString(s, _drawFontCenter);
                _rightBorder = (int)Math.Ceiling(_leftBorder + sf.Width * _colCount);
                _wFF = sf.Width;
                Invalidate();
            }
        }

        private void initGrafik()
        {
            Graphics g = this.CreateGraphics();
            if (_drawFont1 != null) _drawFont1.Dispose();
            _drawFont1 = new Font("Courier New", _fontsize);  // Verdana
            _drawFontCenter = new Font("Verdana", _fontsize);  // Verdana

            SizeF sf = g.MeasureString("FFFFFF:F", _drawFont1); // Leerzeichen gelten nicht
            _leftBorder = (int)sf.Width;
            _rowHeight = (int)(sf.Height * 1.10);  // Zeilenabstand
            String s="F F";
            sf = g.MeasureString(s, _drawFontCenter);
            _rightBorder = (int)Math.Ceiling(_leftBorder + sf.Width*_colCount);
            _wFF = sf.Width;

        }

        private void panel_Paint(object sender, PaintEventArgs e)
        {
            int offset, y;
            Graphics g = e.Graphics;
            Pen pen = new Pen(Color.Black, 1);

            y = 10;
            offset = _offset;

            if (_bFirst) _blockRowCount = 0;
            StringBuilder sb2 = new StringBuilder();
            String s1 = "";

            while ((y + _rowHeight) <= Height)
            {
                // hole Text
                byte[] array = _file.loadRow(offset, _colCount);
                if (array.Length == 0) break;
                g.DrawString(String.Format("{0:X}", offset).PadLeft(6, '0'), _drawFont1, _drawBrush1, 10, y);

                sb2.Clear();
                int i = 0;
                foreach (byte b in array)
                {
                    s1 = String.Format("{0:X}", b).PadLeft(2, '0') + " ";
                    g.DrawString(s1, _drawFontCenter, _drawBrushCenter, (int) (_leftBorder + (_wFF * i)), y);

                    if ((b < 32) || (b == 153))  // Trademark
                        sb2.Append(".");
                    else
                        if (b == 32)  // Trademark
                            sb2.Append("_");
                        else
                            sb2.Append((char)b);
                    i++;
                }

                // zeichnen 
                g.DrawString(sb2.ToString(), _drawFont1, _drawBrush1, _rightBorder, y);

                offset += _colCount;
                y += _rowHeight;
                if (_bFirst) _blockRowCount++;
            }
            if (_bFirst)
            {
                _bFirst = false;
                _vScrollBar1.LargeChange = _blockRowCount;  // Anzahl der Zeilen pro Fenster
            }
            pen.Dispose();
            g.Dispose();
        }


        public void PosTop()
        {
            _offset = 0;
            Invalidate();
        }
        public void PosDown()
        {
            _offset = _vScrollBar1.Maximum;
            Invalidate();
        }      


        public void myKeyDown(int KeyValue)
        {
            int lastrow = _vScrollBar1.Maximum;
            switch (KeyValue)
            {
                case (int)Keys.PageUp: if ((_offset - _blockRowCount * _colCount) >= 0)
                        _offset -= _blockRowCount * _colCount;
                        else
                            _offset = 0;
                        break;
                case (int)Keys.PageDown: if ((_offset + _blockRowCount * _colCount*2) < _filesize)
                            _offset += _blockRowCount * _colCount;
                        else
                        {
                            int rest = (int) (_filesize % _colCount);
                            _offset = (int) (_filesize - rest - (_blockRowCount - 1) * _colCount);
                        }
                        break;

                case (int)Keys.Left: ;// <-
                    break;
                case (int)Keys.Right: ;// ->
                    break;

                case (int)Keys.Up: if (_offset > _colCount)  // Pfeil oben
                        _offset -= _colCount;
                        else
                            _offset = 0;
                        break;

                case (int)Keys.Down: if ((_offset + _blockRowCount * _colCount) <= _filesize)  // Pfeil Down
                                        _offset += _colCount;
                                    else
                                    {
                                        int rest = (int) (_filesize % _colCount);
                                        _offset = (int) (_filesize - rest - (_blockRowCount - 1) * _colCount);
                                    }
                    break;                
            }
            Invalidate();
        }

        private void vScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            _offset= _vScrollBar1.Value*_colCount;
            Invalidate();
        }
        public void saveToFile(String sFileName)
        {
            StringBuilder sStr = new StringBuilder();
            int offset = 0;

            StringBuilder sb1 = new StringBuilder();
            StringBuilder sb2 = new StringBuilder();
            int colCount = 16;

            sStr.Append("Datei: " + sFileName);
            sStr.Append(Environment.NewLine);
            sStr.Append(Environment.NewLine);

            while (true)
            {
                // hole Text
                byte[] array = _file.loadRow(offset, colCount);
                if (array.Length == 0) break;
                sStr.Append( String.Format("{0:X}", offset).PadLeft(6, '0') );
                sb1.Clear();
                sb2.Clear();
                foreach (byte b in array)
                {
                    sb1.Append(String.Format("{0:X}", b).PadLeft(2, '0') + " ");
                    if ((b < 32) || (b == 153))  // Trademark
                        sb2.Append(".");
                    else
                        if (b == 32)  // Trademark
                            sb2.Append("_");
                        else
                            sb2.Append((char)b);
                }
                if (array.Length < _colCount)
                {
                    for (int i = array.Length; i < _colCount; i++) sb1.Append("   ");
                }
                // zeichnen 
                sStr.Append( "  ");
                sStr.Append( sb1.ToString() );
                sStr.Append("  ");
                sStr.Append( sb2.ToString() );
                sStr.Append(Environment.NewLine);
                offset += colCount;
            }
            File.WriteAllText(sFileName, sStr.ToString() );
            Process Prog = new Process();
            Prog.StartInfo.FileName = "notepad.exe";
            Prog.StartInfo.Arguments = sFileName;
            Prog.Start(); 
        }


       
    }
}
